/*
 * Project name:
     nRFControlMaster_pic32.vtft
 * Generated by:
     Visual TFT
 * Date of creation
     01/08/2013
 * Time of creation
     10:33:30
 * Test configuration:
     MCU:             P32MX795F512L
     Dev.Board:       Mikromedia_Plus_for_PIC32MX7
                      http://www.mikroe.com/mikromedia/plus/pic32mx7/
     Oscillator:      80000000 Hz
     SW:              mikroC PRO for PIC32
                      http://www.mikroe.com/mikroc/pic32/
 */

#include "nRFControlMaster_pic32_objects.h"
#include "__Lib_nRF24L01.h"
#include "nRF24L01_defs.h"
#include "led_protocol.h"

sbit nRF_CS at RA4_bit;
sbit nRF_CS_Direction at TRISA4_bit;

sbit nRF_CE at RB14_bit;
sbit nRF_CE_Direction at TRISB14_bit;

sbit nRF_IRQ at RB3_bit;
sbit nRF_IRQ_Direction at TRISB3_bit;

sbit LED_B at RD3_bit;
sbit LED_B_Direction at TRISD3_bit;

sbit LED_G at RD2_bit;
sbit LED_G_Direction at TRISD2_bit;

sbit LED_R at RD1_bit;
sbit LED_R_Direction at TRISD1_bit;

TnRF_Status nRFStatus;
TRF_Config rfConfig;

unsigned char rfState;
volatile unsigned long readPort;

unsigned char oldValue;
unsigned char newValue;

TLed leds[8];

TButton_Round *btns[8];


void InitLeds(void) {
  unsigned char i;

  for (i = 0; i < 8; i++)
  {
    leds[i]._state = 0;
    leds[i]._no    = i;
  }
}

//Initialization of Timer 2
void InitTimer2(){
  // Initialize TIMER2 module for CE pulse
  T2CON             = 0x8000;
  T2IE_bit          = 1;
  T2IF_bit          = 0;
  T2IP0_bit         = 0;
  T2IP1_bit         = 1;
  T2IP2_bit         = 1;
  PR2               = 1280;
  TMR2              = 0;
  ON__T2CON_bit     = 1;
}

//Interrupt function for Timer 2
void Timer2_interrupt() iv IVT_TIMER_2 ilevel 7 ics ICS_SRS {
  T2IF_bit = 0;
  TIMER2_IRQ();
  T2IE_bit = 0;                    // stop timer 2
  ON__T2CON_bit = 0;
}


//Interrupt function for Change Notice Interrupt
void CN5_Interrupt() iv IVT_CHANGE_NOTICE ilevel 7 ics ICS_SRS {
  readPort = PORTB;

  newValue = readPort;

  CNIF_bit = 0;        //clear flag

  // event on falling edge
  if(newValue < oldValue)
    nRF24L01_IRQ();
}

//Initialization of External Interrupt
void InitExt(){
///////////////////////// Change Notify interrupt
  ON__CNCON_bit = 1; // enable Change Notice (CN) module
  CNCONbits.SIDL = 0;

  CNEN5_bit = 1;    // set Change Notice 5

  //CNPUE5_bit = 1;   // set pull up
  readPort = PORTB;

  newValue = readPort;
  oldValue = readPort;

  CNIP0_bit = 1;    // set priority to lvl 7
  CNIP1_bit = 1;
  CNIP2_bit = 1;

  CNIF_bit = 0;
  CNIE_bit  = 1;    // enable interrupt*/

}

void InitRF()
{
  LED_B_Direction = 0;
  LED_B = 0;

  LED_G_Direction = 0;
  LED_G = 0;

  LED_R_Direction = 0;
  LED_R = 0;

  nRF_CE = 0;
  nRF_CS = 1;

  DisableInterrupts();
  //Initialize timer and external interrupt
  InitTimer2();
  InitExt();
  EnableInterrupts(); // enable all interrupts
  //Initialize SPI1
  SPI1_Init();
}

void main() {
  btns[0] = &ButtonRound8;
  btns[1] = &ButtonRound7;
  btns[2] = &ButtonRound6;
  btns[3] = &ButtonRound5;
  btns[4] = &ButtonRound4;
  btns[5] = &ButtonRound3;
  btns[6] = &ButtonRound2;
  btns[7] = &ButtonRound1;

  InitLeds();

  //nRF Config Settings
  rfConfig.autoTransmitDelay = nRF_AUTO_RETRANSMIT_DELAY_4000us; //250 + 15 * 250 = 4000us
  rfConfig.autoTransmitCount = nRF_AUTO_RETRANSMIT_COUNT_05;
  rfConfig.channel           = 64;
  rfConfig.outputPower       = nRF_OUTPUT_POWER_0dBm;
  rfConfig.dataRate          = nRF_DATA_RATE_1Mbps;

  Start_TP();

  nRF_Init();
  InitRF();
  Delay_ms(100);

  rfState = RF_STOP;

  nRF_TX_Mode_Init_Advanced(rfConfig.dataRate, rfConfig.autoTransmitDelay, rfConfig.autoTransmitCount, rfConfig.outputPower, rfConfig.channel);

  while (1) {
    Check_TP();

    readPort = PORTB;
    oldValue = readPort;
    newValue = readPort;

    if(rfState == RF_SEND_CMD)
    {
      if(nRF_Send_Packet())
      {
        LED_R = ~LED_R;
        Delay_ms(50);
        nRF_Process(&nRFStatus);

        if(nRFStatus._status == nRF_TX_DS)
        {
          LED_B = ~LED_B;
          LED_R = 0;

          rfState = RF_STOP;
        }
      }
    }
  }
}
